#!/usr/bin/env python

import argparse
from datetime import datetime
from restapidoc.restapidoc import parse, group_by_module, doc_template
from restapidoc.config import modules as module_description
from mako import exceptions
import codecs

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--coverage", help="ouputs into stdin documentation coverage",
                        action="store_true", default=False)
    parser.add_argument("-d", "--dir", help="folder in which we need look for documentation annotation. default: test_src_folder/",
                        default="test_src_folder/")
    parser.add_argument("--filter", help="gob filter in which files to search. default: *Controler*.php",
                        default="*Controller*.php")
    parser.add_argument("-o", "--output", help="output filename. default: documentation.html",
                        default="documentation.html")
    args = parser.parse_args()

    blocks = parse(args.dir, args)
    grouped_blocks = group_by_module(blocks)
    try:
        output = doc_template.render(blocks=blocks, grouped_blocks=grouped_blocks,
                                     when=datetime.today(), modules=module_description)

        with codecs.open(args.output, "wb", encoding='utf-8') as f:
            f.write(output)
    except:
        print(exceptions.text_error_template().render())

if __name__ == '__main___':
    main()


modules = {}

modules['calls'] = """
Using the Twilio REST API, you can make outgoing calls to phones, SIP-enabled endpoints and Twilio Client connections.

Note that calls initiated via the REST API are rate-limited to one per second. You can queue up as many calls as you like as fast as you like, but each call is popped off the queue at a rate of one per second.
"""

modules['tweets'] = """
Tweets are the atomic building blocks of Twitter, 140-character status updates with additional associated metadata. People tweet for a variety of reasons about a multitude of topics.
"""


import re

action = re.compile(r"\s+public function (\w+)Action\(", re.S | re.I)

docblock = re.compile(r"((?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/))", re.M | re.S)
cleanup = re.compile(r"^\s+\*(.*?)$", re.M)

route = re.compile(r"^@route (.*)$", re.M)
ingroup = re.compile(r"^@(?:ingroup|group) (.*)$", re.M)
method = re.compile(r"^@method (.*)$", re.M)

returns = re.compile(r"^@(?:json_return|return|returns) (.*)$", re.M)

nonworking = re.compile(r"^@(notworking|deprecated) (.*)$", re.M)

# returns four groups: param_type, argname, type, description
params = re.compile(r"^@(?P<param_type>url|json|get|post)_?params? (?P<arg>[\w\.]+)(?: (?P<type>\w+)(?: (?P<else>.*?))?)?$", re.M | re.I)

examples = re.compile(r"@example (?P<type>\w+)\s*(?P<example>^(?:(?!@example).)*)", re.I | re.S | re.M)

module = re.compile(r"modules/(\w+)/controllers/", re.I)



import expressions as e
import re
from config import modules as module_description
import hashlib


class Param:
    def __init__(self, param):
        self.paramtype, self.name, self.type, self.rest = param
        self.required = False
        if "required" in self.rest.lower() and "not required" not in self.rest.lower():
            self.required = True
            self.rest = self.rest.replace("required", "")
        self.rest = self.rest.replace("not required", "")


class Example:
    def __init__(self, example):
        example_type, example_code = example
        self.type = example_type
        self.code = example_code

    def getcode(self):
        return self.code.encode("latin-1", errors='replace')


class Docblock:
    def __init__(self, block=""):
        self.file = ""
        self.module = ""
        self.route = ""
        self.method = ""
        self.group = ""
        self.examples = []
        self.params = []
        self.description = ""
        self.filename = ""
        self.returns = ""
        self.block = block
        self.working = True
        self.deprecated = False
        self.non_working = ""

        self.is_api_doc = False
        self.is_api_doc = self.parse()

    def parse(self):
        """
        parses dockblock and returns true if it's rest API dockblock
        false if it's not
        """

        if "@method" not in self.block or "@route" not in self.block:
            return False

        self.block = self.cleanup_docblock()

        self.route = self._get_route()
        self.method = self._get_method()
        self.group = self._get_group()
        self.returns = self._get_returns()
        self.params = self._get_params()
        self.non_working = self._get_non_working()

        self.examples = self._get_examples()

        # rest is considered to be as description
        self.description = self.block.strip()

        return True

    def uniq(self):
        return hashlib.sha224(self.method+self.route+self.getgroup()).hexdigest()

    def is_danger(self):
        if not self.working or self.deprecated:
            return "danger"
        return ""

    def cleanup_docblock(self):
        docblock = e.cleanup.sub(r"\1", self.block)
        docblock = re.sub(r"^ ", "", docblock, 0, re.M)
        return docblock[3:-2]

    def getgroup(self):
        if self.group != "":
            return self.group.lower()
        return self.module.lower()

    def get_module_desc(self):
        if self.getgroup() in module_description.keys():
            return module_description[self.getgroup()]
        return ""

    def _get_route(self):
        finds = e.route.findall(self.block)
        if len(finds) > 0:
            self.remove_code_block(e.route)
            return finds[0]
        return ""

    def _get_method(self):
        finds = e.method.findall(self.block)
        if len(finds) > 0:
            self.remove_code_block(e.method)
            return finds[0]
        return [0]

    def _get_group(self):
        finds = e.ingroup.findall(self.block)
        if len(finds) > 0:
            self.remove_code_block(e.ingroup)
            return finds[0]
        return ""

    def _get_returns(self):
        find = e.returns.findall(self.block)
        if len(find) > 0:
            self.remove_code_block(e.returns)
            return find[0]
        return ""

    def _get_params(self):
        found_params = e.params.findall(self.block)

        if len(found_params) > 0:
            self.remove_code_block(e.params)
            return [Param(p) for p in found_params]

        return []

    def _get_non_working(self):
        f = e.nonworking.findall(self.block)
        if len(f) > 0:
            self.remove_code_block(e.nonworking)
            t, m = f[0]
            if t.lower().strip() == "deprecated":
                self.deprecated = True
            else:
                self.working = False
            return m
        return ""

    def _get_examples(self):
        f = e.examples.findall(self.block)
        if len(f) > 0:
            self.remove_code_block(e.examples)
            return [Example(c) for c in f]
        return []

    def remove_code_block(self, pattern):
        self.block = pattern.sub("", self.block)

    def __str__(self):
        return "%s %s (%s)" % (self.method.upper(), self.route, self.group)



# -*- coding: utf-8 -*-
import os
import codecs
import fnmatch
import expressions as e
from models import Docblock

from mako.lookup import TemplateLookup

template_lookup = TemplateLookup(directories=[os.path.join(os.path.dirname(__file__), 'templates/')],
                                 input_encoding='utf-8', output_encoding='utf-8',
                                 encoding_errors='replace')

doc_template = template_lookup.get_template("doc.html")


def get_controllers(folder, args):
    for root, dirnames, filenames in os.walk(folder):
        for filename in fnmatch.filter(filenames, args.filter):
            yield os.path.join(root, filename)


def how_many_actions(code):
    return len(e.action.findall(code))


def get_actions(code):
    return e.action.findall(code)


def get_docblock(code):
    return e.docblock.findall(code)


def group_by_module(blocks):
    r = {}
    for b in blocks:
        group = b.getgroup()
        if group not in r.keys():
            r[group] = []
            r[group].append(b)
        else:
            r[group].append(b)
    return r


def parse(folder, args):
    methods_count = 0
    documentation_count = 0

    found_blocks = []
    for controller_filename in get_controllers(folder, args):
        f = codecs.open(controller_filename, "r", encoding='utf-8')
        code = "".join(f.readlines())
        f.close()

        methods_count += how_many_actions(code)

        blocks = get_docblock(code)

        if len(blocks) > 0:
            for b in blocks:
                module_name = e.module.findall(controller_filename)
                parsed_block = Docblock(b)
                parsed_block.filename = controller_filename

                if parsed_block.is_api_doc:
                    if len(module_name) > 0:
                        parsed_block.module = module_name[0]

                    found_blocks.append(parsed_block)
                    documentation_count += 1

    if args.coverage and methods_count > 0:
        print("Service documentation coverage")
        print("{0} service methods found and {1} documented. Coverage: {2}%".format(
              methods_count, documentation_count, documentation_count * 1. / methods_count * 100))

    return found_blocks


from __future__ import print_function
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import codecs
import os
import sys
import re

here = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    # intentionally *not* adding an encoding option to open
    return codecs.open(os.path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

long_description = read('README.md')

setup(
    name='restapidoc',
    version=find_version('restapidoc', '__init__.py'),
    url='https://github.com/dzhibas/rest-api-doc/',
    author='Nikolajus Krauklis',
    install_requires=['Mako>=0.8',],
    author_email='nikolajus@gmail.com',
    description='REST APIs documentation builder tool',
    long_description=long_description,
    include_package_data=True,
    package_data = {
        'restapidoc': ['templates/*', 'README.md'],
    },
    packages=find_packages(exclude=['test_src_folder', 'test_src_folder.*']),
    entry_points={
        'console_scripts': [
            'restapidoc = restapidoc.utils.runner:main',
        ],
    },
    platforms='any',
    classifiers = [
        'Programming Language :: Python',
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Topic :: Utilities',
        ]
)
